/*
 * IBM Confidential OCO Source Materials
 * 5639-D57, 5630-A36, 5630-A37, 5724-D18 (C) Copyright IBM Corp. 1992, 2004
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 */

#ifndef HTAPI_H
#define HTAPI_H

#include <time.h>  /* for time_t */
#include "htctypes.h"

/*
 * Return values from user-written functions
 * Refer to the Web Programming Guide and HTTP specification for details.
 */

/* server specific return codes */
#define HTTP_AUTHORIZATION_STOP_REQUEST -2
#define HTTP_REQUEST_SERVICED_LOG -1
#define HTTP_NOACTION              0

/* 1xx Informational */
#define HTTP_CONTINUE            100
#define HTTP_SWITCHING_PROTOCOLS 101

/* 2xx Successful */
#define HTTP_OK                  200
#define HTTP_CREATED             201
#define HTTP_ACCEPTED            202
#define HTTP_NON_AUTHORITATIVE   203
#define HTTP_NO_CONTENT          204
#define HTTP_RESET_CONTENT       205
#define HTTP_PARTIAL_CONTENT      206
#define HTTP_PARTIAL_CONENT      HTTP_PARTIAL_CONTENT

/* 3xx Redirection */
#define HTTP_MULTIPLE_CHOICES    300
#define HTTP_MOVED_PERMANENTLY   301
#define HTTP_FOUND               302
#define HTTP_MOVED_TEMPORARILY   HTTP_FOUND
#define HTTP_SEE_OTHER           303
#define HTTP_NOT_MODIFIED        304
#define HTTP_USE_PROXY           305
/* 306 is unused */
#define HTTP_TEMPORARY_REDIRECT  307

/* 4xx Client Error */
#define HTTP_BAD_REQUEST         400
#define HTTP_UNAUTHORIZED        401
#define HTTP_PAYMENT_REQUIRED    402
#define HTTP_FORBIDDEN           403
#define HTTP_NOT_FOUND           404
#define HTTP_METHOD_NOT_ALLOWED  405
#define HTTP_NOT_ACCEPTABLE      406
#define HTTP_PROXY_UNAUTHORIZED  407
#define HTTP_REQUEST_TIMEOUT     408
#define HTTP_CONFLICT            409
#define HTTP_GONE                410
#define HTTP_LENGTH_REQUIRED     411
#define HTTP_PRECONDITION_FAILED 412
#define HTTP_ENTITY_TOO_LARGE    413
#define HTTP_URI_TOO_LONG        414
#define HTTP_BAD_MEDIA_TYPE      415
#define HTTP_BAD_RANGE           416
#define HTTP_EXPECTATION_FAILED  417

/* 5xx Server Error */
#define HTTP_SERVER_ERROR        500
#define HTTP_NOT_IMPLEMENTED     501
#define HTTP_BAD_GATEWAY         502
#define HTTP_SERVICE_UNAVAILABLE 503
#define HTTP_GATEWAY_TIMEOUT     504
#define HTTP_BAD_VERSION         505

/* Return values from server provided functions */
#define HTTPD_UNSUPPORTED        -1
#define HTTPD_SUCCESS             0
#define HTTPD_FAILURE             1
#define HTTPD_INTERNAL_ERROR      2
#define HTTPD_PARAMETER_ERROR     3
#define HTTPD_STATE_CHECK         4
#define HTTPD_READ_ONLY           5
#define HTTPD_BUFFER_TOO_SMALL    6
#define HTTPD_AUTHENTICATE_FAILED 7
#define HTTPD_EOF                 8
#define HTTPD_ABORT_REQUEST       9
#define HTTPD_REQUEST_SERVICED    10
#define HTTPD_RESPONSE_ALREADY_COMPLETED   11
#define HTTPD_WRITE_ONLY          12

#define HTTPD_HTC_IN_USE          13
#define HTTPD_HTC_READ_ONLY	  14
#define HTTPD_HTC_WRITE_ONLY	  15
#define HTTPD_HTC_NOT_VARIANT	  16
#define HTTPD_HTC_OFFLINE	  17
#define HTTPD_HTC_FILE_EXISTS	  18
#define HTTPD_HTC_DEVICE_EXISTS	  19
#define HTTPD_HTC_NO_DEVS	  20
#define HTTPD_HTC_TOO_MANY_DEVS	  21
#define HTTPD_HTC_NO_TMP_CNTRS	  22
#define HTTPD_HTC_BAD_BLOCK_SIZE  23
#define HTTPD_HTC_BAD_LO_WATER	  24
#define HTTPD_HTC_BAD_HI_WATER	  25
#define HTTPD_HTC_BAD_DEVICE	  26
#define HTTPD_HTC_BAD_CNTR_SIZE	  27
#define HTTPD_HTC_TOO_FEW_CNTRS	  28
#define HTTPD_HTC_INVALID_STATE	  29
#define HTTPD_HTC_BAD_LABEL	  30
#define HTTPD_HTC_BAD_DEV_ID	  31
#define HTTPD_HTC_BAD_OFFSET	  32
#define HTTPD_HTC_URI_TOO_LONG	  33
#define HTTPD_HTC_BAD_LENGTH	  34
#define HTTPD_HTC_FILE_NOT_FOUND  35
#define HTTPD_HTC_OUT_OF_MEMORY	  36
#define HTTPD_HTC_CACHE_FULL	  37
#define HTTPD_HTC_BAD_FORMAT	  38
#define HTTPD_HTC_IO_ERROR	  39
#define HTTPD_HTC_TOO_MANY_CNTRS  40
#define HTTPD_HTC_NOT_A_FILE	  41
#define HTTPD_HTC_UNSUPPORTED	  42


/* Arguments for httpd_setvar to manipulate         */
/* http headers...                                  */
/*  replace     - Replace all occurences of the     */
/*                 header with header passed in.    */
/*  add         - Append this header to the headers */
/*  replace_add - Replace the first occurence if    */
/*                 the header exists, otherwise,    */
/*                 add it.                          */
/*  delete_all  - Delete all occurences of this     */
/*                 header.                          */
#define HTTPD_SETVAR_REPLACE      0
#define HTTPD_SETVAR_REPLACE_ADD  1
#define HTTPD_SETVAR_ADD          2
#define HTTPD_SETVAR_REMOVE_ALL   3
#define HTTPD_SETVAR_REMOVE       4

/* prototypes for server provided functions */

#if defined(__OS2__)
#define HTTPD_LINKAGE _System
#elif defined(WIN32)
#define HTTPD_LINKAGE __stdcall
#else
#define HTTPD_LINKAGE
#endif

/* Long name to short name mapping */
#define HTTPD_authenticate      HTAUTHEN
#define HTTPD_extract           HTXTRACT
#define HTTPD_binextract        HTBINXTRACT
#define HTTPD_set               HTSET
#define HTTPD_binset            HTBINSET
#define HTTPD_replace           HTREPLACE
#define HTTPD_encode            HTENC
#define HTTPD_decode            HTDEC
#define HTTPD_file              HTFILE
#define HTTPD_exec              HTEXEC
#define HTTPD_read              HTREAD
#define HTTPD_write             HTWRITE
#define HTTPD_log_access        HTLOGA
#define HTTPD_log_event         HTLOGV
#define HTTPD_log_error         HTLOGE
#define HTTPD_log_trace         HTLOGT
#define HTTPD_restart           HTREST
#define HTTPD_proxy             HTPROXY
#define HTTPD_noauth_proxy      HTNOAUPXY
#define HTTPD_attributes        HTATTRIB
#define HTTPD_reverse_translate HTREVTRA
#define HTTPD_translate         HTTRANS
#define HTTPD_supply_label      HTSUPLBL
#define httpd_getvar            htgetvar
#define httpd_setvar            htsetvar
#define httpd_getvarint         htgetvarint
#define httpd_setvarint         htsetvarint
#define HTTPD_open              htopen
#define HTTPD_close             htclose
#define httpd_variant_insert    htvins
#define httpd_variant_lookup    htvfind
#define HTTPD_cacheable_url     HTCACHBLE
#define HTTPD_cacheg_invalidate_url	HTCACHEGINV
#define HTTPD_cacheg_url_owner	HTCACHEGOWN

#define HTTPD_cache_peek        HTCAPEEK
#define HTTPD_CC_Lookup         CCLOOKUP
#define HTTPD_CC_Create         CCCREATE
#define HTTPD_CC_Read           CCREAD
#define HTTPD_CC_Write          CCWRITE
#define HTTPD_CC_Delete         CCDELETE
#define HTTPD_CC_Invalidate     CCINVAL
#define HTTPD_CC_Lock           CCLOCK
#define HTTPD_CC_Unlock         CCUNLOCK
#define HTTPD_CC_Swap           CCSWAP
#define HTTPD_CC_GetConfig      CCGCONF 
#define HTTPD_CC_GetAttr        CCGATTR
#define HTTPD_CC_SetAttr        CCSATTR
#define HTTPD_CC_Dis_CDF	HTTPD_CC_Dis_CDF
#define HTTPD_CC_Ena_CDF	HTTPD_CC_Ena_CDF
#define HTTPD_CC_WriteHeaders	HTTPD_CC_WriteHeaders
#define HTTPD_CC_WriteData	HTTPD_CC_WriteData
#define HTTPD_getFileUrlMappingCursor  HTGETCUR
#define HTTPD_dropFileUrlMappingCursor HTDROPCUR
#define HTTPD_insertFileUrlMapping     HTINSCUR

#define HTTPD_create_async_handle 	HTCREATASYN
#define HTTPD_start_req		HTSTARTHTTPR
#define HTTPD_wait_async_handle	HTWAITASYN
#define HTTPD_delete_async_handle	HTDELASYN

#define HTTPD_urlrewrite               HTURLRW
#define HTTPD_addrwtag                 HTADDRWTAG
#define HTTPD_removerwtag              HTREMRWTAG

#define HTTPD_inv_url_rewrite          HTINVURLRW
#define HTTPD_junctionrewritesetcookie HTCOOKIERW

/* The exposed memory pool interface */
#define HTTPD_MP_malloc                HTMPMALLOC
#define HTTPD_MP_calloc                HTMPCALLOC
#define HTTPD_MP_strdup                HTMPSTRDUP
#define HTTPD_MP_strcat		       HTMPSTRCAT
#define HTTPD_MP_free		       HTMPFREE		/* No need to call in plugin, will free automally */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef enum _CA_PEEKRES {
          PEEK_NO = 0, /* Not found in the cache */
          PEEK_FOUND = 1  /* Cache hit, the request will servered from cache */
} CA_PEEKRES;

typedef struct _htclient_buf {
    char *req_bufP;     /* I: The request buffer */
    char *req_bodyP;	/* I: The request body */
    char *res_bufP;	/* IO: The response buffer */
    int  res_bufsize; /* I: The length of the response buffer */
    void *asyncHandle;  /* I: async handle */
    unsigned int  *res_lenP; /* O: The length returned in response buffer */
    long *retCodeP;    /* O: The error code returned */
} htclient_buf_t;

/* authenticate a userid/password                */
/* valid only in PreExit, Authentication, and Authorization steps */
void
HTTPD_LINKAGE
HTTPD_authenticate(
             unsigned char *handle,  /* i; handle  */
             long *return_code);     /* o; return code */

/* extract the value of a variable associated with this request */
/* valid in all steps, though not all variables will be         */
void
HTTPD_LINKAGE
HTTPD_extract(
    unsigned char *handle,       /* i; handle  */
    unsigned char *name,         /* i; variable to extract */
    unsigned long *name_length,  /* i; length of the name */
    unsigned char *value,        /* o; buffer in which to place the value */
    unsigned long *value_length, /* i/o; size of buffer/length of value */
             long *return_code); /* o; return code */

/* extract the value of a user defined variable associated with this request */
/* valid in all steps                                                        */
void
HTTPD_LINKAGE
HTTPD_binextract(
    unsigned char *handle,       /* i; handle  */
    unsigned char *name,         /* i; variable to extract */
    unsigned long *name_length,  /* i; length of the name */
    void *value,        /* o; buffer in which to place the value */
    unsigned long *value_length, /* i/o; size of buffer/length of value */
             long *return_code); /* o; return code */


/* replace the nth_header occurance value of a variable associated */
/* with this request                                               */
/* If value or value_length is NULL, it will simply delete the old */
/* header.                                                         */
/* No effect if the variable can not be found                      */
void
HTTPD_LINKAGE
HTTPD_replace(
     unsigned char *handle,      /* i; handle  */   
     unsigned char *name,        /* i; variable to be replaced  */
     unsigned long *name_len,    /* i; length of the name       */
     unsigned char *value,       /* i; new value  */
     unsigned long *value_size,  /* i; length of the new value  */
     unsigned long *nth_header,  /* i; nth occurance of the variable */
              long *return_code);

/* set the value of a variable associated with this request     */
/* the available variables are the same as CGI                  */
/* - note that one can create variables with this function      */
/* - if created variables are prefixed by "HTTP_", they will    */
/*   be sent as headers in the response, without the "HTTP_"    */
/*   prefix                                                     */
/* valid in all steps, though not all variables are             */
void
HTTPD_LINKAGE
HTTPD_set(
    unsigned char *handle,       /* i; handle  */
    unsigned char *name,         /* i; variable to set */
    unsigned long *name_length,  /* i; length of the name */
    unsigned char *value,        /* o; buffer which contains the value */
    unsigned long *value_length, /* i; length of value */
             long *return_code); /* o; return code */

/* set the value of a user defined variable associated with this request */
/* valid in all steps                                                    */
void
HTTPD_LINKAGE
HTTPD_binset(
    unsigned char *handle,       /* i; handle  */
    unsigned char *name,         /* i; variable to set */
    unsigned long *name_length,  /* i; length of the name */
    void *value,        /* o; buffer which contains the value */
    unsigned long *value_length, /* i; length of value */
             long *return_code); /* o; return code */



/* send a file to satisfy this request */
/* valid only in PreExit, Service, Error and Datafilter steps  */
void
HTTPD_LINKAGE
HTTPD_file(
    unsigned char *handle,       /* i; handle  */
    unsigned char *name,         /* i; name of file to send */
    unsigned long *name_length,  /* i; length of the name */
             long *return_code); /* o; return code */

/* run a CGI script to satisfy this request */
/* valid only in PreExit, Service and Error steps  */
void
HTTPD_LINKAGE
HTTPD_exec(
    unsigned char *handle,       /* i; handle  */
    unsigned char *name,         /* i; name of script to execute */
    unsigned long *name_length,  /* i; length of the name */
             long *return_code); /* o; return code */


/* Encode a single line of binary data to a standard format that */
/* uses only printing ASCII characters (but takes up 33% more bytes */  
/*    Entry    bufin    points to a buffer of bytes.  If nbytes is not
 *                      a multiple of three, then the byte just beyond
 *                      the last byte in the buffer must be 0.
 *             nbytes   is the number of bytes in that buffer.
 *                      This cannot be more than 48. This must be a multiple
 *                      of 3.
 *             bufcoded points to an output buffer.  Be sure that this
 *                      can hold at least 1 + (4*nbytes)/3 characters.
 *
 *    Exit     bufcoded contains the coded line.  The first 4*nbytes/3 bytes
 *                      contain printing ASCII characters representing
 *                      those binary bytes. This may include one or
 *                      two '=' characters used as padding at the end.
 *                      The last byte is a zero byte.
 */

void
HTTPD_LINKAGE
HTTPD_encode(
    unsigned char *handle,
    unsigned char *bufin,
    unsigned int  nbytes,
    unsigned char *bufcoded,
             long *return_code);


/* Decode an ASCII-encoded buffer back to its original binary form.
 *   Entry     bufcoded    points to a uuencoded string.  It is
 *                         terminated by any character not in
 *                         the printable character table six2pr, but
 *                         leading whitespace is stripped.
 *             bufplain    points to the output buffer; must be big
 *                         enough to hold the decoded string (generally
 *                         shorter than the encoded string) plus
 *                         as many as two extra bytes used during
 *                         the decoding process.
 *             outbufsize  is the maximum number of bytes that
 *                         can fit in bufplain.
 *
 *    Exit     Returns the number of binary bytes decoded.
 *             bufplain    contains these bytes. 
 */

void 
HTTPD_LINKAGE
HTTPD_decode(
    unsigned char *handle,
             char *bufcoded,          
    unsigned char *bufplain,
             int  outbufsize,
    long *return_code);


/* peek cache to see if the specified url is going to be servered from */
/* cache or not.                                                       */
/* Return value: PEEK_NO      document is not found in cache          */
/*               PEEK_FOUND   document will be servered from cache    */
CA_PEEKRES
HTTPD_LINKAGE
HTTPD_cache_peek(
    unsigned char *handle,       /* i; handle */
    unsigned char *url,          /* i; name of url */
    unsigned long *urlLen,       /* i; length of the url */
             long *return_code); /* o; return code */



/* read the body of the client's request - use set/extract for headers */
/* Keep reading until HTTPD_EOF is returned; 4k is a good buffer size  */
/* valid only in the PreExit, Service, and Datafilter steps  */
void
HTTPD_LINKAGE
HTTPD_read(
    unsigned char *handle,       /* i; handle  */
    unsigned char *value,        /* i; buffer in which to place the value */
    unsigned long *value_length, /* i/o; size of buffer/length read */
             long *return_code); /* o; return code */

/* write the body of the response - use set/extract for headers */
/* valid only in PreExit, Service, Error and Datafilter steps  */
void
HTTPD_LINKAGE
HTTPD_write(
    unsigned char *handle,       /* i; handle  */
    unsigned char *value,        /* i; data to send */
    unsigned long *value_length, /* i; length of the data */
             long *return_code); /* o; return code */

/* write a string to the server's access log */
/* valid in all steps */
void
HTTPD_LINKAGE
HTTPD_log_access(
    unsigned char *handle,       /* i; handle  */
    unsigned char *value,        /* i; data to write */
    unsigned long *value_length, /* i; length of the data */
             long *return_code); /* o; return code */

/* write a string to the server's event log */
/* valid in all steps */
void
HTTPD_LINKAGE
HTTPD_log_event(
    unsigned char *handle,       /* i; handle  */
    unsigned char *value,        /* i; data to write */
    unsigned long *value_length, /* i; length of the data */
             long *return_code); /* o; return code */


/* write a string to the server's error log */
/* valid in all steps */
void
HTTPD_LINKAGE
HTTPD_log_error(
    unsigned char *handle,       /* i; handle  */
    unsigned char *value,        /* i; data to write */
    unsigned long *value_length, /* i; length of the data */
             long *return_code); /* o; return code */

/* write a string to the server's trace log */
/* valid in all steps */
void
HTTPD_LINKAGE
HTTPD_log_trace(
    unsigned char *handle,       /* i; handle  */
    unsigned char *value,        /* i; data to write */
    unsigned long *value_length, /* i; length of the data */
             long *return_code); /* o; return code */


/* restart the server after all active requests have been processed */
/* valid in all steps except ServerInit and ServerTerm */
void
HTTPD_LINKAGE
HTTPD_restart(long *return_code); /* o; return code */

/* make a proxy request */
/* valid in PreExit and Service steps  */
/* Note: This is a completion function; i.e, the response is complete after */
/*       this function returns */
void
HTTPD_LINKAGE
HTTPD_proxy(
    unsigned char *handle,       /* i; handle  */
    unsigned char *url_name,     /* i; URL to request */
    unsigned long *name_length,  /* i; length of the URL */
             void *request_body, /* i; body of the request */
    unsigned long *body_length,  /* i; length of the body */
             long *return_code); /* o; return code */

/* make a proxy request without calling nametrans/auth */
/* valid in authorization step */
/* Note: This is a completion function; i.e, the response is complete after */
/*       this function returns */
void
HTTPD_LINKAGE
HTTPD_noauth_proxy(
    unsigned char *handle,       /* i; handle  */
    unsigned char *url_name,     /* i; URL to request */
    unsigned long *name_length,  /* i; length of the URL */
             void *request_body, /* i; body of the request */
    unsigned long *body_length,  /* i; length of the body */
             long *return_code); /* o; return code */

/* Check a URL to see if it is cacheable by the Proxy or not */
void
HTTPD_LINKAGE
HTTPD_cacheable_url(
    unsigned char *handle,       /* i; handle  */
    unsigned char *url,     /* i; URL to request */
    unsigned char *req_method, /*client request method*/
             long *retval); /* o; return code */


/* NOT SUPPORTED */
void
HTTPD_LINKAGE
HTTPD_attributes(
    unsigned char *handle,
    unsigned char *name,
    unsigned long *name_length,
    unsigned char *value,
    unsigned long *value_length,
             long *return_code);

/* NOT SUPPORTED */
void
HTTPD_LINKAGE
HTTPD_reverse_translate(
    unsigned char *handle,
    unsigned char *name,
    unsigned long *name_length,
    unsigned char *value,
    unsigned long *value_length,
             long *return_code);

/* NOT SUPPORTED */
void
HTTPD_LINKAGE
HTTPD_translate(
    unsigned char *handle,
    unsigned char *name,
    unsigned long *name_length,
    unsigned char *url_value,
    unsigned long *url_value_length,
    unsigned char *path_trans,
    unsigned long *path_trans_length,
    unsigned char *query_string,
    unsigned long *query_string_length,
             long *return_code);

/* Use this function in a PicsDBLookup exit to supply labels */
void
HTTPD_LINKAGE
HTTPD_supply_label(
    unsigned char *handle,       /* i; handle */
    unsigned char *value,        /* i; PICS label being supplied */
    unsigned long *value_size,   /* i; length of the PICS label */
             long *return_code); /* o; return code */

/* 'C' friendly verison of HTTPD_extract() */
const
unsigned char *            /* o; value of variable */
HTTPD_LINKAGE
httpd_getvar(
    unsigned char *handle, /* i; handle */
    unsigned char *name,   /* i; variable name */
    unsigned long *n);     /* i; get nth occurence of this header */

/* 'C' friendly version of HTTPD_set() */
long                       /* o; return code */
HTTPD_LINKAGE
httpd_setvar(
    unsigned char *handle, /* i; handle */
    unsigned char *name,   /* i; variable name */
    unsigned char *value,  /* i; new value */
    unsigned long *addHdr);/* i; manipulate headers, see SETVAR macros above */

int
HTTPD_LINKAGE
httpd_setvarint(
                unsigned char *handle,
                unsigned char *name,
                unsigned int value );

int
HTTPD_LINKAGE
httpd_getvarint(
                unsigned char *handle,
                unsigned char *name,
                unsigned int *value );

/* Transmogrifier utility */
void
HTTPD_LINKAGE
HTTPD_open (
    unsigned char *handle,       /* i; handle */
             long *return_code); /* o; return code */

/* Transmogrifier utility */
void
HTTPD_LINKAGE
HTTPD_close (
    unsigned char *handle,       /* i; handle */
             long *return_code); /* o; return code */

/* This function is 'C' friendly */
/* Insert a variant into the cache */
void
HTTPD_LINKAGE
httpd_variant_insert(
    unsigned char *handle,       /* i; handle */
    unsigned char *URI,          /* i; URI of this object */
    unsigned char *dimension,    /* i; dimension this object varies in */
    unsigned char *variant,      /* i; value of the variant */
    unsigned char *filename,     /* i; file containing the object */
             long *return_code); /* o; return code */

/* This function is 'C' friendly */
/* Determine if a given variant exists in the cache */
void
HTTPD_LINKAGE
httpd_variant_lookup(
    unsigned char *handle,       /* i; handle */
    unsigned char *URI,          /* URI of this object */
    unsigned char *dimension,    /* i; dimension this object varies in */
    unsigned char *variant,      /* i; value of the variant */
             long *return_code); /* o; return code */

void
HTTPD_LINKAGE
HTTPD_cacheg_invalidate_url(
    unsigned char *handle,      /* i; handle */
    unsigned char *URI,         /* URI of this object */
             long *return_code);/* o; return code */
			    
void HTTPD_LINKAGE HTTPD_cacheg_url_owner(
	unsigned char *handle, /* i; handle */
	unsigned char *url,    /* i; URI of object */	
        unsigned char **owner,  /* o; owner(peer) if RCA defined	*/
	long *retval);	       /* o; return code */


/* URL rewrite interfaces */

/* Function: HTTPD_addrwtag()
 * Purpose:  To allow user to be able to add new tag and properies into rewrite
 *           parser.
 * Return Values:
*/
void HTTPD_LINKAGE HTTPD_addrwtag(
        unsigned char *handle,
        long          *retval,...);


/* Function: HTTPD_removerwtag()
 * Purpose:  To allow user to be able to add new tag and properies into rewrite
 *           parser.
 * Return Values:
*/
void HTTPD_LINKAGE HTTPD_removerwtag(
        unsigned char *handle,
        long *retval,...);



/* Function: HTTPD_urlrewrite()
 * Purpose:  To allow user to be able to rewrite the imbeded
 *           "server relative" url.  This function is extremely
 *           useful for reverse proxy mode.
 * In:       buf -- original data
 *           buflen -- original data length
 *           prefix_pat -- the pattern
 * Out:      outBufP -- rewritten data. Space allocated by proxy. Caller
 *                      should not try to free it.
 *           outBuflen -- rewritten data length.
 * Return Values:
 *           HTTPD_SUCCESS -- has been rewritten, use data in outBufP
 *           Otherwise     -- has not been rewritten, use data in buf
 */
void HTTPD_LINKAGE HTTPD_urlrewrite(
        unsigned char *handle,
        unsigned char *buf,
        unsigned long *buflen,
        unsigned char *prefix_pat,
        unsigned char **outBufP,
        unsigned long *outBuflen,
        long          *retval);

void HTTPD_LINKAGE HTTPD_junctionrewritesetcookie(
        unsigned char *handle,
        long          *retval);

/* 
 * CCAPI routines
 */
void HTTPD_LINKAGE HTTPD_CC_Lookup(
				   unsigned char *url,
				   unsigned long *urlLength,
				   long *isValid,
				   long *retval
				   );


void HTTPD_LINKAGE HTTPD_CC_Create(
				   unsigned char *url,
				   unsigned long *urlLength,
				   void **fileHandle,
				   time_t *lastModified,
				   time_t *maxUnused,
				   time_t *expires,
				   long *retval
				   );

void HTTPD_LINKAGE HTTPD_CC_Read(
				 void *fileHandle,
				 char *dataBuf,
				 unsigned long *dataLength,
				 char *headerBuf,
				 unsigned long *headerLength,
				 long *retval
				 );

void HTTPD_LINKAGE HTTPD_CC_Write(
				  void *fileHandle,
				  char *dataBuf,
				  unsigned long *dataLength,
				  char *headerBuf,
				  unsigned long *headerLength,
				  long *isClFlag,
				  long *retval
				  );

void HTTPD_LINKAGE HTTPD_CC_Delete(
				   unsigned char *url,
				   unsigned long *urlLength,
				   long *retval
				   );

void HTTPD_LINKAGE HTTPD_CC_Invalidate(
				       unsigned char *url,
				       unsigned long *urlLength,
				       long *retval
				       );

void HTTPD_LINKAGE HTTPD_CC_Lock(
				 unsigned char *url,
				 unsigned long *urlLength,
				 long *lockMode,
				 void **fileHandle,
				 long *isValid,
				 long *retval
				 );

void HTTPD_LINKAGE HTTPD_CC_Unlock(
				   void *fileHandle,
				   long *lockMode,
				   long *retval
				   );

void HTTPD_LINKAGE HTTPD_CC_Swap(
				 void *fileHandle1,
				 void *fileHandle2,
				 long *retval
				 );

void HTTPD_LINKAGE HTTPD_CC_GetConfig(
				      void *apiInfo
				      );

void HTTPD_LINKAGE HTTPD_CC_GetAttr(
				    void *fileHandle,
				    HTTPD_CC_Stat_t *stat,
				    unsigned long *flags,
				    long *retval
				    );

void HTTPD_LINKAGE HTTPD_CC_SetAttr(
				    void *fileHandle,
				    HTTPD_CC_Stat_t *stat,
				    unsigned long *flags,
				    long *retval
				    );

void HTTPD_LINKAGE HTTPD_CC_Dis_CDF( long *return_code);

void HTTPD_LINKAGE HTTPD_CC_Ena_CDF(long *return_code);

void HTTPD_LINKAGE HTTPD_CC_WriteData(
				      void *fileHandle,
				      char *dataBuf,
				      unsigned long *dataLength,
				      long *retval
				      );

void HTTPD_LINKAGE HTTPD_CC_WriteHeaders(
					 void *fileHandle,
					 char *headerBuf,
					 unsigned long *headerLength,
					 long *isClFlag,
					 long *retval
					 );

void * HTTPD_LINKAGE HTTPD_getFileUrlMappingCursor(
						   char *cdn,
                                                   char *object
						   );


void HTTPD_LINKAGE HTTPD_dropFileUrlMappingCursor(
						  char *cdn,
                                                  char *key
						  );


void HTTPD_LINKAGE HTTPD_insertFileUrlMapping(
					      char *cdn,
                                              char *filename,
                                              char *url
					      );

/* ASYNC req handling routines */

void HTTPD_LINKAGE HTTPD_create_async_handle(
	unsigned char *handle,			
	void **hndlPP,
	long *retval);

void HTTPD_LINKAGE HTTPD_start_req(
	unsigned char *handle,
        void *httpEnv,				   
        void *asyncHandle,
	long *retval);

void HTTPD_LINKAGE HTTPD_wait_async_handle(
	unsigned char *handle,
	void *hndlPP,
	long *retval);

void HTTPD_LINKAGE HTTPD_delete_async_handle(
	unsigned char *handle,
	void *hndlPP,
	long *retval);


char*
HTTPD_LINKAGE
HTTPD_inv_url_rewrite(
            unsigned char *handle,
            char *url);

void*
HTTPD_LINKAGE
HTTPD_MP_malloc(unsigned char* handle, int size);

void*
HTTPD_LINKAGE
HTTPD_MP_calloc(unsigned char* handle, int num, int size);

char*
HTTPD_LINKAGE
HTTPD_MP_strdup(unsigned char* handle, char* str);

char*
HTTPD_LINKAGE
HTTPD_MP_strcat(unsigned char* handle, char* str1, char* str2);

void
HTTPD_LINKAGE
HTTPD_MP_free(unsigned char* handle, void* p);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* HTAPI_H */
