/*
IBM Confidential

OCO Source Materials

5630-A36

(c) Copyright IBM Corp. 1992, 2002

The source code for this program is not published or other-
wise divested of its trade secrets, irrespective of what has
been deposited with the U.S. Copyright Office.
*/

/*
 * Websphere Edge Server Cache Control API public types file: htctypes.h
 *
 */

#ifndef _htctypes_h
#define _htctypes_h

/*
 * As you implement more optional interfaces, tack them on at the end of this
 * list, and update the CCAPIOptApis macro in htcapi.h
 */
#define CC_Lock        0x0001
#define CC_Unlock      0x0002
#define CC_Swap        0x0004

typedef struct {
    int optApis; /* optional APIs: conjunct of above optional interface bits */
    int majorVersion;
    int minorVersion;
    /* more */
} CCAPIInfo_t;

/*
 * Lock modes
 */
#define HTTPD_CC_LOCK_SHARED    0
#define HTTPD_CC_LOCK_EXCLUSIVE 1


/*
 * Structure used for cache file stats.
 * This was taken from the HTCFsOps_Stat_t structure
 * definition in server/htcache/htcfsops.h. Keep them in sync.
 */
typedef struct HTTPD_CC_Stat_s {
    time_t expires;		/* Expiration time */
    time_t last_modified;	/* Modification time */
    time_t last_access;		/* Last access time */
    time_t max_unused;		/* Maximum time to keep unused file */
    time_t sent_time;		/* Time object was sent */
    unsigned long size;		/* File size in bytes including headers */
    unsigned char flags;	/* Flags from file header. */
    unsigned char load_delay;	/* Load delay for GC calculations */
    unsigned short usage_count; /* Number of accesses */
} HTTPD_CC_Stat_t;


/*
 * Flags used to selectively set file stats. 
 * This was taken from server/htcache/htcfsops.h.
 * Most of the stats are read/only. Keep them in sync.
 */
#define HTTPD_CC_STAT_EXPIRES	        0x1	/* set expires */
#define HTTPD_CC_STAT_LAST_MODIFIED	0x2	/* set last_modified */
#define HTTPD_CC_STAT_LAST_ACCESS	0x4	/* set last_access */
#define HTTPD_CC_STAT_MAX_UNUSED	0x8	/* set maxtime to keep unused file */
#define HTTPD_CC_STAT_SENT_TIME	        0x10	/* set sent_or_locked_time */
#define HTTPD_CC_STAT_LOAD_DELAY	0x20	/* set load delay */
#define HTTPD_CC_STAT_USAGE_COUNT	0x40	/* set usage count */
#define HTTPD_CC_STAT_STATE		0x80    /* set file state */
#define HTTPD_CC_STAT_ADD_VALUES        0x100   /* add instead of setting values */

#endif  /* _htctypes_h */
