/*
     This is a sample demo plugin that will work with Junction Rewrite plugin to parse the html pages.
     Currently there are 2 handlers for APPLET, and SCRIPT tag block. Please change the code to get the
     resource link maped. 
 */

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "HTAPI.h"
#include "junctionRewrite2.h"


static char* MODRW_CONTENT_TYPE[] = { "text/html", "text/css", NULL};

typedef struct _interestingBlock {
        char* tag;					/* the tag that is interesting */
        long (*handler)(unsigned char* handle,		/* the handler of the block    */
                        unsigned char* prefix,		/* The prefix need to be added */
                        unsigned char* input, 		/* The input stream buffer     */
                        unsigned long  inSize,		/* The input buffer size       */
                        unsigned char** outputpp, 	/* The output buffer pointer's address */
                        unsigned long* outSizep,	/* The output buffer size pointer */
                        unsigned long* outCountp);	/* The current size of output buffer */
} HTInterestingBlock;

HTInterestingBlock IBlocks[] = { 
                   {"script",           script_handler}, 
                   {"applet",           applet_handler},
                   {NULL, NULL}
};

unsigned char* is_a_interestingTag(unsigned char* tagsp, unsigned char* tagep) {
       if(!tagsp) return NULL;
       if(*tagsp == '<') tagsp++;
       while(WHITESPACE(*tagsp)) tagsp++;
       if (*tagsp) {
          long i = 0;
          while(IBlocks[i].tag != NULL) {
                if(!strncasecmp(IBlocks[i].tag, tagsp, strlen(IBlocks[i].tag))) return IBlocks[i].tag;
                i++;
          }
       }
       return NULL;
}


void* httpd_getVoidPointer(unsigned char* handle,
                           unsigned char* name) {
	long n = 0;
        void* addr = NULL;
	const unsigned char* addrString = httpd_getvar(handle, name, &n);
        if (addrString) {
            sscanf(addrString, "%x", &addr);
        }
	return addr;
}

void httpd_setVoidPointer(unsigned char* handle,
                          unsigned char* name,
                          void* addr) {	char buf[64];
        unsigned long n = HTTPD_SETVAR_REPLACE;
	sprintf(buf, "%x", addr);
	httpd_setvar(handle, name, buf, &n);
}

long httpd_getLongValue(unsigned char* handle,
                           unsigned char* name) {
        long n = 0;
        long lv = 0;
        const unsigned char* addrString = httpd_getvar(handle, name, &n);
        if (addrString) {
            sscanf(addrString, "%x", &lv);
        }
        return lv;
}

void httpd_setLongValue(unsigned char* handle,
                        unsigned char* name,
                        long lv) {
        char buf[64];
        unsigned long n = HTTPD_SETVAR_REPLACE;
        sprintf(buf, "%x", lv);
        httpd_setvar(handle, name, buf, &n);
}

void *
HTTPD_LINKAGE
rewrite2_open (unsigned char *handle,
               long          *retc) {

    unsigned long n = 0;
    int ix, do_rewrite = FALSE;
    unsigned char *valid_ctypeP;
    unsigned char *prefix = NULL;
    unsigned char *content_type = NULL;
    unsigned char trace_string[1024];
    long tracestrlen, rc;

    *retc = HTTP_NOACTION;
    prefix = (unsigned char*)httpd_getvar(handle, "_THP_MODRW_PREFIX", &n);
    if (NULL != prefix) {
        content_type = (unsigned char*)httpd_getvar(handle, "PROXY_CONTENT_TYPE", &n);
        if (content_type != NULL) {
            /* clean up string */
            while (content_type && WHITESPACE(*content_type)) content_type++;
            for (ix = 0, valid_ctypeP = MODRW_CONTENT_TYPE[0]; valid_ctypeP; ix++, valid_ctypeP = MODRW_CONTENT_TYPE[ix]) {
                if (strncasecmp(content_type, valid_ctypeP, strlen(valid_ctypeP)) == 0) {
                    do_rewrite = TRUE;
                    break;
                }
            }

            /* Prefix there but response is not rewritable */
            sprintf(trace_string, "rewrite2: %s URL for this content (%s)\n",
                    do_rewrite == TRUE ? "Rewriting" : "Will not rewrite", content_type);
            tracestrlen = strlen(trace_string);
            HTTPD_log_trace(handle, trace_string, &tracestrlen, &rc);
            if (do_rewrite) {
                prefix = strdup(prefix);	/* Allocate the space of the prefix */
                httpd_setVoidPointer(handle, "REWRITE2_PREFIX", prefix);

                httpd_setLongValue(handle, "REWRITE2_STATE", REWRITE2_STATE_START);
                httpd_setLongValue(handle, "REWRITE2_STATE_2", STATE_NONE);
                httpd_setLongValue(handle, "REWRITE2_SPECIAL", 0);

                httpd_setVoidPointer(handle, "REWRITE2_BUFFER", NULL);
                httpd_setLongValue(handle, "REWRITE2_BUFFER_SIZE",  0);
                httpd_setLongValue(handle, "REWRITE2_BUFFER_COUNT", 0);
                httpd_setLongValue(handle, "REWRITE2_BUFFER_OFFSET", 0);

                httpd_setVoidPointer(handle, "REWRITE2_OUTPUT", NULL);
                httpd_setLongValue(handle, "REWRITE2_OUTPUT_SIZE",  0);
                httpd_setLongValue(handle, "REWRITE2_OUTPUT_COUNT",  0);

                httpd_setVoidPointer(handle, "REWRITE2_INTEREST", NULL);

                HTTPD_open(handle, &rc);
                *retc = HTTP_OK;
            }
        }
    }
    return(NULL);
}


int
HTTPD_LINKAGE
rewrite2_write (unsigned char *handle,
        unsigned char *data,
        unsigned long *length,
        void          *correlator,
        long          *retc) { 
        long rc;
        unsigned char*  rewrite2prefix;

        rewrite2prefix = httpd_getVoidPointer(handle, "REWRITE2_PREFIX");
        if (rewrite2prefix) {
             rc = rewrite2_handle(handle, data, length, rewrite2prefix);
        } else {
             HTTPD_write(handle, data, length, &rc);
        }

        *retc = HTTP_OK;
        return;

}

void
HTTPD_LINKAGE
rewrite2_close (unsigned char *handle,
        void          *correlator,
        long          *retc) {

    long rc = 0, length = 0;
    unsigned char* rewrite2prefix, *datap, *outputp, *interestTag;

    /* flush buffered stream, pass NULL as the input stream.  */
    rewrite2prefix = httpd_getVoidPointer(handle, "REWRITE2_PREFIX");
    rc = rewrite2_handle(handle, NULL, &length, rewrite2prefix);

    datap         = httpd_getVoidPointer(handle, "REWRITE2_BUFFER");
    outputp       = httpd_getVoidPointer(handle, "REWRITE2_OUTPUT");
    interestTag   = httpd_getVoidPointer(handle, "REWRITE2_INTEREST");
    rewrite2prefix = httpd_getVoidPointer(handle, "REWRITE2_PREFIX");

    if(datap) free(datap);
    if(outputp) free(outputp);
    if(interestTag) free(interestTag);
    if(rewrite2prefix) free(rewrite2prefix);

    HTTPD_close(handle, retc);

    *retc = HTTP_OK;
    return;
}

void
HTTPD_LINKAGE
rewrite2_error (unsigned char *handle,
        void          *correlator,
        long          *retc)
{

    *retc = HTTP_OK;
    return;
}

int WRITE_OUTPUT(unsigned char* handle, 
                 unsigned char** outputpp, 
                 unsigned long* outSizep, 
                 unsigned long* outCountp, 
                 unsigned char c) {

        unsigned char* outputp;
        unsigned long  outSize;
        unsigned long  outCount;
        unsigned char* tempp;
        long rc;

        if (handle == NULL || outputpp == NULL || outSizep == NULL || outCountp == NULL) {
            return 0;
        }

	outputp =  *outputpp;
        outCount = *outCountp;
        outSize  = *outSizep;

        if(outputp == NULL) {
            tempp = (unsigned char*)malloc(BUFFER_BASE_SIZE);
            if(tempp) {
               outputp  = tempp;
               outCount = 0;
               outSize  = BUFFER_BASE_SIZE;
            } else {
               long n = 1;
               HTTPD_write(handle, &c, &n, &rc);
               goto EXIT;
            }
        }

        if (outCount >= outSize - 1) {
            if(outSize < BUFFER_LIMIT) {
                tempp = realloc(outputp, outSize + BUFFER_BASE_SIZE);
                if(tempp) {
                   outputp = tempp;
                   outSize += BUFFER_BASE_SIZE;
                } 
            }

            if (outCount >= outSize - 1) {
               HTTPD_write(handle, outputp, &outCount, &rc);
               outCount = 0;
            }
        }

        *(outputp + outCount) = c;
        outCount++;

EXIT:
        *outputpp = outputp;
        *outSizep = outSize;
        *outCountp = outCount;
        return 1;
}

int FLUSH_OUTPUT(unsigned char* handle, 
                 unsigned char** outputpp, 
                 unsigned long* outSizep,  
                 unsigned long* outCountp) {

        unsigned char* outputp;
        unsigned long  outSize;
        unsigned long  outCount;
        long rc, sz;

        if (handle == NULL || outputpp == NULL || outSizep == NULL || outCountp == NULL) {
            return 0;
        }

        outputp =  *outputpp;
        outCount = *outCountp;
        outSize  = *outSizep;

        if(!outputp) return 0;
        if (outCount > 0) {
            HTTPD_write(handle, outputp, &outCount, &rc);
        }
        sz = outCount;
        outCount = 0;

        *outputpp = outputp;
        *outSizep = outSize;
        *outCountp = outCount;
        return sz;
}


long rewrite2_handle(unsigned char* handle,
                     unsigned char* data, 
                     unsigned long* lengthp, 
                     unsigned char* prefix) {

       unsigned long length = *lengthp;
       unsigned long i = 0, n = 0, rc = 0;
       unsigned long count = 0;
       unsigned long bufSize, bufCount = 0, bufOffset = 0;
       unsigned long outSize, outCount = 0;
       unsigned char* tempp;
       unsigned char *outputp, *writep;
       unsigned char *datap, *currentp;
       unsigned char *tagstartp = NULL, *tagendp = NULL;
       unsigned char *intereststartp, *interestendp, *interestingTag; 
       long special, rewrite2state, state;

        /* Recover the states from previous call of this function */
        rewrite2state = httpd_getLongValue(handle, "REWRITE2_STATE");
        state         = httpd_getLongValue(handle, "REWRITE2_STATE_2");
        special       = httpd_getLongValue(handle, "REWRITE2_SPECIAL");

        datap         = httpd_getVoidPointer(handle, "REWRITE2_BUFFER");
        bufSize       = httpd_getLongValue(handle, "REWRITE2_BUFFER_SIZE");
        bufCount      = httpd_getLongValue(handle, "REWRITE2_BUFFER_COUNT");
        bufOffset     = httpd_getLongValue(handle, "REWRITE2_BUFFER_OFFSET");

        outputp       = httpd_getVoidPointer(handle, "REWRITE2_OUTPUT");
        outSize       = httpd_getLongValue(handle, "REWRITE2_OUTPUT_SIZE");
        outCount      = httpd_getLongValue(handle, "REWRITE2_OUTPUT_COUNT");

        interestingTag   = httpd_getVoidPointer(handle, "REWRITE2_INTEREST");

        /* If rewrite is aborted, we just write the data down stream */
        if (rewrite2state == REWRITE2_STATE_ABORT) {
            if(datap != NULL) {
               HTTPD_write(handle, datap, &bufCount, &rc);
               free(datap);
               datap = NULL;
            }
            if (data) {
                HTTPD_write(handle, data, &length, &rc);
            }
            httpd_setVoidPointer(handle, "REWRITE2_BUFFER", NULL);
            httpd_setLongValue(handle, "REWRITE2_BUFFER_SIZE",  0);
            httpd_setLongValue(handle, "REWRITE2_BUFFER_COUNT",  0);
            httpd_setLongValue(handle, "REWRITE2_BUFFER_OFFSET",  0);
            return bufCount + length;
        }

        /* If buffer the input stream, if it is too small, do not handle it just return 0. */
	if (data != NULL) {
            if (datap == NULL) {
                bufSize  = BUFFER_BASE_SIZE;
                datap = malloc(bufSize);
                if(!datap) {
                    rewrite2state = REWRITE2_STATE_ABORT;
                    HTTPD_write(handle, data, &length, &rc);
                    httpd_setVoidPointer(handle, "REWRITE2_BUFFER", NULL);
                    httpd_setLongValue(handle, "REWRITE2_BUFFER_SIZE",  0);
                    httpd_setLongValue(handle, "REWRITE2_BUFFER_COUNT",  0);
                    httpd_setLongValue(handle, "REWRITE2_BUFFER_OFFSET",  0);
                    httpd_setLongValue(handle, "REWRITE2_BUFFER_STATE", rewrite2state);
                    return length;
                } else {
                    bufCount = 0;
                }
            }

            if (bufCount + length >= bufSize) {
                tempp = realloc(datap, bufSize + (length << 1) );
                if(!tempp) {
                    HTTPD_write(handle, datap, &bufCount, &rc);
                    free(datap);
                    datap = NULL;
                    rewrite2state = REWRITE2_STATE_ABORT;
                    if(data) {
                        HTTPD_write(handle, data, &length, &rc);
                    }
                    httpd_setVoidPointer(handle, "REWRITE2_BUFFER", NULL);
                    httpd_setLongValue(handle, "REWRITE2_BUFFER_SIZE",  0);
                    httpd_setLongValue(handle, "REWRITE2_BUFFER_COUNT",  0);
                    httpd_setLongValue(handle, "REWRITE2_BUFFER_OFFSET",  0);
                    httpd_setLongValue(handle, "REWRITE2_BUFFER_STATE", rewrite2state);
                    return bufCount + length;
                } else {
                    datap = tempp;
                    bufSize += (length << 1);
                }
            }
            memcpy(datap + bufCount, data, length);
            bufCount += length;
            *(datap + bufCount) = 0;
            if (length < 256 && bufCount < BUFFER_LIMIT) {
                httpd_setVoidPointer(handle, "REWRITE2_BUFFER", datap);
                httpd_setLongValue(handle, "REWRITE2_BUFFER_SIZE", bufSize);
                httpd_setLongValue(handle, "REWRITE2_BUFFER_COUNT", bufCount);
                httpd_setLongValue(handle, "REWRITE2_BUFFER_OFFSET", bufOffset);
                return 0;
            }
        }

        /* Now, start parse and handle the buffered page */
        currentp = datap + bufOffset;
        count    = 0;
        length   = bufCount - bufOffset;
        if( rewrite2state == REWRITE2_STATE_INTEREST) {
            intereststartp = datap;
            interestendp = NULL;
        } else {
            intereststartp = NULL;
            interestendp = NULL;
        }

        while(count < length) {
             
            switch(rewrite2state) {
                case REWRITE2_STATE_START:
                    if(special) { special = 0; WRITE_OUTPUT(handle, &outputp, &outSize, &outCount, *currentp); goto NEXT; }
                    if(SPECIAL(*currentp)) { special = 1; WRITE_OUTPUT(handle, &outputp, &outSize, &outCount, *currentp); goto NEXT; }
                    if(*currentp == '<') { state = STATE_LEFT; tagstartp = currentp; goto NEXT; }
                    if(*currentp == '>' && tagstartp != NULL) {
                       state = STATE_RIGHT;
                       tagendp = currentp;
                       if (interestingTag = is_a_interestingTag(tagstartp, tagendp)) {
                           interestingTag = strdup(interestingTag);
                           intereststartp = tagstartp;
                           rewrite2state = REWRITE2_STATE_INTEREST;
                           goto NEXT;
                       } else {
                           /* Not an ineresting tag, output this tag */
                           while(tagstartp <= tagendp) {
                               WRITE_OUTPUT(handle, &outputp, &outSize, &outCount, *tagstartp);
                               tagstartp++;
                           }
                           tagstartp = tagendp = NULL;
                           state = STATE_NONE;
                           goto NEXT;
                       }
                    } else if(tagstartp == NULL) {
                        WRITE_OUTPUT(handle, &outputp, &outSize, &outCount, *currentp);
                    }
                    break;
                 case REWRITE2_STATE_INTEREST:
                    if(special) { special = 0; goto NEXT; }
                    if(SPECIAL(*currentp)) { special = 1; goto NEXT; }
                    if(*currentp == '<') { state = STATE_LEFT; tagstartp = currentp; goto NEXT; }
                    if(*currentp == '>') {
                        if(*(tagstartp+1) == '/' && !strncasecmp((tagstartp+2), interestingTag, strlen(interestingTag))) {
                             /* Is a closure of an interesting tag block */
                             interestendp = currentp;
                             i = 0;
                             while (IBlocks[i].tag) {
                                 if (!strncasecmp(interestingTag, IBlocks[i].tag, strlen(interestingTag))) {
                                     IBlocks[i].handler(handle, prefix,
                                                        intereststartp, interestendp - intereststartp, 
                                                        &outputp, &outSize, &outCount);
                                     rewrite2state = REWRITE2_STATE_START;
                                     intereststartp = interestendp = NULL;
                                     free(interestingTag);
                                     interestingTag = NULL;
                                     break;
                                 }
                                 i++;
                             }
                        }
                        tagstartp = tagendp = NULL;
                        state = STATE_NONE;
                    }
                    break;
             }
NEXT:
             count++;
             currentp++;
        }

        /* If we need flush and the page is truncated page, flush the local strings */
        if (data == NULL) {
            /* called by stream close function */
            if (intereststartp) {
                while(intereststartp < currentp) {
                    WRITE_OUTPUT(handle, &outputp, &outSize, &outCount, *intereststartp);
                    intereststartp++;
                }
                intereststartp = interestendp = NULL;
            } else if(tagstartp) {
                while(tagstartp < currentp) {
                    WRITE_OUTPUT(handle, &outputp, &outSize, &outCount, *tagstartp);
                    tagstartp++;
                }
                tagstartp = tagendp = NULL;
            }
        }
    
        /* flush the output */
        if( outSize) {
            FLUSH_OUTPUT(handle, &outputp, &outSize, &outCount);
        }

        /* No more data, save the states */
        if (intereststartp) {
            bufCount = currentp - intereststartp;
            i = 0;
            if(intereststartp != datap) {
                while(i < bufCount) {
                    *(datap + i) = *(intereststartp + i); 
                    i++;
                }
            }
            *(datap + bufCount) = 0;
            if(tagstartp) {
                bufOffset = tagstartp - intereststartp;
            } else {
                bufOffset = bufCount;
            }
            state = STATE_NONE;
        } else if(tagstartp) {
            bufOffset = 0;
            bufCount = currentp - tagstartp;
            i = 0;
            if(tagstartp != datap) {
                while(i < bufCount) {
                    *(datap+i) = *(tagstartp+i);
                    i++;
                }
            }
            *(datap + bufCount) = 0;
            state = STATE_NONE;
        } else {
            /* nothing need to buffer, all put into output stream */
            state = STATE_NONE;
            bufOffset = 0;
            bufCount  = 0;
        }

        
        httpd_setLongValue(handle, "REWRITE2_STATE", rewrite2state);
        httpd_setLongValue(handle, "REWRITE2_STATE_2", STATE_NONE);
        httpd_setLongValue(handle, "REWRITE2_SPECIAL", special);

        httpd_setVoidPointer(handle, "REWRITE2_BUFFER", datap);
        httpd_setLongValue(handle, "REWRITE2_BUFFER_SIZE",  bufSize);
        httpd_setLongValue(handle, "REWRITE2_BUFFER_COUNT", bufCount);
        httpd_setLongValue(handle, "REWRITE2_BUFFER_OFFSET", bufOffset);

        httpd_setVoidPointer(handle, "REWRITE2_OUTPUT", outputp);
        httpd_setLongValue(handle, "REWRITE2_OUTPUT_SIZE",  outSize);
        httpd_setLongValue(handle, "REWRITE2_OUTPUT_COUNT",  outCount);

        httpd_setVoidPointer(handle, "REWRITE2_INTEREST", interestingTag);
}


